define([], function() {
	var $ = Framework7.$;

	var history = [];
	var lastDepth = 0;
	var numOverflow = 0;


	/**
	 * Init router, that handle page events
	 */
    function init() {
		$(document).on('pageBeforeInit', function (e) { onEvent('init', e); });
		//$(document).on('pageInit', function (e) { onEvent('onInit', e); });
		//$(document).on('pageReinit', function (e) { onEvent('reInit', e); });
		//$(document).on('pageBeforeAnimation', function (e) { onEvent('beforeAnimation', e); });
		$(document).on('pageAfterAnimation', function (e) { onEvent('afterAnimation', e); });
		$(document).on('pageBeforeRemove', function (e) { onEvent('unload', e); });
		$(document).on('pageBack', function (e) { onEvent('beforeBack', e); });
		//$(document).on('pageAfterBack', function (e) { onEvent('afterBack', e); });
    }


	function getHistory(reset) {
		var data = { history: history, now: (Date.now() / 1000)|0, numOverflow: numOverflow };

		if (typeof reset !== undefined && reset) {
			history = [];
			numOverflow = 0;
		}

		return data;
	}

	function getMainController() {
		if (app.Device.hasInitialized()) {
			/* When the device is a tablet and in landscape mode the main view should display the 'home' page, otherwise it should display the menu. */
			return app.Device.isTablet() && app.Device.hasOrientation('landscape') ? 'home' : 'menu';
		} else {
			return false;
		}
	}

	function getQueryParameter(name, url) {
		if (!url) url = window.location.href;
		name = name.replace(/[\[\]]/g, "\\$&");
		var regex = new RegExp("[?&]" + name + "(=([^&#]*)|&|#|$)"),
		    results = regex.exec(url);
		if (!results) return null;
		if (!results[2]) return '';
		return decodeURIComponent(results[2].replace(/\+/g, " "));
	}

	function onEvent(action, e) {
		var page = validPage(e);
		var view = $(page.container).closest('.view').data('view');
		if (page) route(action, page.name, page.query, view);
	}

	function onPageBack(page) {
		if (app.isWebView && app.mainView.history.length <= 2) {
			$('.webview-overlay').show();
			window.history.back();
		}
	}

	function onPageLoaded(page, query) {
		switch (page) {
		case 'home':
		case 'login':
		case 'menu':
			/* do nothing */
			break;

		default:
			if (app.hasWebOverlay) $('.webview-overlay').hide();
		}
	}

	function logHistory(page, context) {
		var back = false;
		var item;

		/* Page history is only appended when first loading a page. The code below
		 * prevents the history from being appended when going "back" through pages
		 * in the view history (even though they may be getting reloaded for redisplay).
		 */
		if (app.mainView.history.length < lastDepth) back = true;
		lastDepth = app.mainView.history.length;
		if (back) return;

		switch (page) {
		case 'home':
		case 'login':
		case 'menu':
			return;

		case 'benefits':
		case 'info':
			page += '-' + (context && context.id ? context.id : 0);
			context = false;
			break;
		}

		if (history.length < 100) {
			item = { p: page, d: (Date.now() / 1000)|0 };
			if (typeof context !== 'undefined' && context) item.c = context;

			history[history.length] = item;
		} else {
			numOverflow++;
		}
	}

	function validPage(e) {
		var page = e.detail.page;

		/* Don't attempt to load a controller for F7 internal objects (e.g. smart selects) */
		return (!page.url || page.url.indexOf('#') !== 0) ? page : false;
	}

	/**
	 * Load (or reload) controller from js code (another controller) - call it's init function
	 *
	 * @param controllerName
	 * @param query
	 */
	function route(action, controllerName, query, view) {
		controllerName = controllerName === 'main' ? getMainController() : controllerName;
		if (!controllerName) return;

		require(['page/' + controllerName + '/'+ controllerName + 'Controller'], function(controller) {
			switch (action) {
			case 'afterAnimation':
				onPageLoaded(controllerName, query);
				logHistory(controllerName, query);
				break;
			case 'beforeBack':
				onPageBack(controllerName);
				break;
			}

			if (controller[action]) controller[action](query, view);
		});
	}

	return {
        init: init,
		getHistory: getHistory,
		getQueryParameter: getQueryParameter,
		logHistory: logHistory,
		route: route
    };
});
